#
# __init__.py
#
# Created by Nguyen Quang Duy on Sat Dec 16 2023
#

import os
import requests
import numpy as np
import pandas as pd
import pyarrow as pa
import collections
from typing import List
from deepvision.common import Service
from deepvision.common.typed_dict import AllOptionsDict
from deepvision.chroma_manager import ChromaDbManager
from typing import List


class FaceClient(Service):

    def __init__(self, api_key: str, domain: str, port: str,):
        """Init service"""
        super().__init__(api_key)
        self.available_services = []
        self.metric_type = "cosine"
        self.url_embedding = "http://10.0.1.247:8090/v1/urlencoding"
        self.url_detect = "http://10.0.1.247:8090/v1/det"
        self.chromadb_manager = ChromaDbManager()

    def get_available_functions(self) -> List[str]:
        """
        Get List of available functions in service
        :return:
        """
        return self.available_services

    def create_collection(self, collection_name):
        try:
            if self.chromadb_manager.check_collection_exists(collection_name):
                return 'Collection %s is already exists' % collection_name
            else:
                collection = self.chromadb_manager.create_collection(name=collection_name,
                                                                     metadata={"hnsw:space": self.metric_type})
                return collection
        except Exception as e:
            raise TypeError(e)

    def remove_item(self, collection_name, remove_id):
        try:
            if self.chromadb_manager.check_collection_exists(collection_name):
                collection = self.chromadb_manager.load_collection(
                    collection_name)
                collection.delete(
                    where={'image_id': remove_id}
                )
                return 'Remove face  %s complete' % remove_id
            else:
                return 'Collection %s does not exists' % collection_name
        except Exception as e:
            raise TypeError(e)

    def remove_subject(self, collection_name, remove_id):
        try:
            if self.chromadb_manager.check_collection_exists(collection_name):
                collection = self.chromadb_manager.load_collection(
                    collection_name)
                collection.delete(
                    where={'subject_id': remove_id}
                )
                return 'Remove subject  %s complete' % remove_id
            else:
                return 'Collection %s does not exists' % collection_name
        except Exception as e:
            raise TypeError(e)

    def remove_all_subject(self, collection_name):
        try:
            if self.check_collection_exists(collection_name):
                collection = self.chromadb_manager.get_collection(
                    name=collection_name)
                search_results = collection.query(
                    query_embeddings=[1]*512,
                    n_results=51157,
                )
                for i in range(0, 51157, 40000):
                    collection.delete(
                        ids=search_results['ids'][0][i:i+40000]
                    )

                return "Delete all subject completed."
            else:
                return 'Collection %s does not exists' % collection_name
        except Exception as e:
            raise TypeError(e)

    def add_data(self, collection_name,
                 subject_ids, subject_names, image_ids, embeddings):
        try:
            data = pd.DataFrame({
                "image_id": image_ids, "subject_id": subject_ids,
                "subject_name": subject_names
            })

            collection = self.chromadb_manager.load_or_create_collection(
                collection_name)

            collection.delete(
                where={'image_id': image_ids}
            )
            metadata = data.to_dict('records')

            collection.add(
                embeddings=embeddings,
                metadatas=metadata,
                ids=data['image_id'],
            )
            return ('Add  complete.')
        except Exception as e:
            raise TypeError(e)

    def add_item(self, collection_name,
                 subject_id, subject_name, image_id, url_image):
        try:
            payload = {'url_image': url_image}
            headers = {}
            response = requests.request(
                "POST", self.url_embedding, headers=headers, data=payload)
            data = pd.DataFrame({
                "image_id": image_id, "subject_id": subject_id,
                "subject_name": subject_name
            })

            collection = self.chromadb_manager.load_or_create_collection(
                collection_name)
            collection.delete(
                where={'image_id': image_id}
            )
            metadata = data.to_dict('records')

            collection.add(
                embeddings=response.data.uploaded_faces,
                metadatas=metadata,
                ids=data['image_id'],
            )
            return ('Add item complete.')
        except Exception as e:
            raise TypeError(e)

    def add_data_from_csv(self, collection_name, csv_dir):
        for single_file in os.listdir(csv_dir):
            df = pd.read_csv(csv_dir + single_file)
            embeddings = [eval(vec) for vec in df.vector.values]
            data = pd.DataFrame({
                "image_id": df['imageID'], "subject_id": df["subjectID"],
                "subject_name": df["subjectName"]
            })
            collection = self.chromadb_manager.load_or_create_collection(
                collection_name)
            collection.delete(
                where={'image_id': df['imageID']}
            )
            metadata = data.to_dict('records')
            collection.add(
                embeddings=embeddings,
                metadatas=metadata,
                ids=data['image_id'].values.tolist(),
            )

    def rename_subject(self, collection_name,
                       subject_id, new_subject_name):
        try:
            if self.chromadb_manager.check_collection_exists(collection_name):
                collection = self.chromadb_manager.load_collection(
                    collection_name)

                search_results = collection.query(
                    query_embeddings=[1]*512,
                    n_results=self.chromadb_manager.count_row_of_collection(
                        collection_name),
                    where={'subject_id': subject_id}
                )

                collection.update(
                    ids=search_results['ids'][0],
                    metadatas=[{
                        'subject_name': new_subject_name}] * len(search_results['ids'][0])
                )
                return 'Rename completed.'
            else:
                return 'Collection %s does not exists' % collection_name
        except Exception as e:
            raise TypeError(e)

    def verify(self, collection_name, subject_id, source_image, target_image):
        try:
            payload = {'source_image': source_image,
                       'target_image': target_image}
            headers = {}
            response = requests.request(
                "POST", self.url_embedding, headers=headers, data=payload)

            if self.chromadb_manager.check_collection_exists(collection_name):
                collection = self.chromadb_manager.load_collection(
                    collection_name)

                search_result = collection.query(
                    query_embeddings=response.data.uploaded_faces,
                    n_results=1,
                )

                if (1 - search_result[0]['_distance'] > 0.52):
                    return {'result': 'True'}
                else:
                    return {'result': 'False'}
            else:
                return 'Collection %s does not exists' % collection_name
        except Exception as e:
            raise TypeError(e)

    def count_1(self, collection_name, search_embeddings):
        try:
            # payload = {'base64_image': base64_image}
            # headers = {}
            # response = requests.request(
            #     "POST", self.url_embedding, headers=headers, data=payload)
            # search_embeddings = response.data.uploaded_faces

            if self.chromadb_manager.check_collection_exists(collection_name):
                search_results = []
                subject_names = []
                results = []
                collection = self.chromadb_manager.load_collection(
                    collection_name)

                search_results = collection.query(
                    query_embeddings=search_embeddings,
                    n_results=1,
                )
                # for embedding in search_embeddings:
                #     search_result = collection.search(embedding) \
                #         .metricType(self.metric_type) \
                #         .limit(1) \
                #         .to_list()
                #     search_results.append(search_result)

                for search_result in search_results:
                    simility = 1 - search_result[0]['_distance']
                    if simility > 0.52:
                        subject_names.append(search_results[0]['subject_name'])
                        dict_res = {
                            'label': search_results[0]['subject_name'],
                            'score': simility
                        }
                        results.append(dict_res)
                    else:
                        subject_names.append("Unknow")
                        dict_res = {'label': "Unknow", 'score': simility}
                result_dicts = {'count': dict(
                    collections.Counter(subject_names)), 'locate': results}
                return result_dicts
            else:
                return 'Collection %s does not exists' % collection_name
        except Exception as e:
            raise TypeError(e)

    def count(self, collection_name, base64_image):
        try:
            payload = {'base64_image': base64_image}
            headers = {}
            response = requests.request(
                "POST", self.url_embedding, headers=headers, data=payload)
            search_embeddings = response.data.uploaded_faces
            det_results = response.data.det_results

            if self.chromadb_manager.check_collection_exists(collection_name):
                search_results = []
                subject_names = []
                # results = []
                collection = self.chromadb_manager.load_collection(
                    collection_name)
                search_results = collection.query(
                    query_embeddings=search_embeddings,
                    n_results=1,
                )
                # for embedding in search_embeddings:
                #     search_result = collection.search(embedding) \
                #         .metricType(self.metric_type) \
                #         .limit(1) \
                #         .to_list()
                #     search_results.append(search_result)

                size_seach_results = len(search_results)
                for i in range(size_seach_results):
                    simility = 1 - search_results[i][0]['_distance']
                    if simility > 0.52:
                        subject_names.append(
                            search_results[i][0]['subject_name'])
                        det_results[i]['score'] = simility
                        det_results[i]['label'] = search_results[i][0]['subject_name']
                    else:
                        subject_names.append("Unknow")
                        det_results[i]['score'] = simility
                        det_results[i]['label'] = 'Unknow'
                result_dicts = {'count': dict(collections.Counter(
                    subject_names)), 'locate': det_results}
                return result_dicts
            else:
                return 'Collection %s does not exists' % collection_name
        except Exception as e:
            raise TypeError(e)

    def detect(self, base64_image):
        try:
            payload = {'base64_image': base64_image}
            headers = {}
            response = requests.request(
                "POST", self.url_detect, headers=headers, data=payload)
            return response
        except Exception as e:
            raise TypeError(e)

    def embedding(self, base64_image):
        try:
            payload = {'base64_image': base64_image}
            headers = {}
            response = requests.request(
                "POST", self.url_embedding, headers=headers, data=payload)
            return response
        except Exception as e:
            raise TypeError(e)
