#
# __init__.py
#
# Created by Nguyen Quang Duy on Sat Dec 16 2023
#

import os
import requests
import numpy as np
import pandas as pd
import pyarrow as pa
import collections
from typing import List
from deepvision.common import Service
from deepvision.common.typed_dict import AllOptionsDict
from deepvision.chroma_manager import ChromaDbManager
from deepvision.client.product.product_embedding import ProductEmbeddingClient


class ProductClient(Service):
    "Product Service"""

    _instance = None  # Class variable to store the instance

    def __new__(cls, api_key: str, domain: str, port: str, options: AllOptionsDict = {}):
        """Create or return the single instance"""
        if cls._instance is None:
            cls._instance = super(ProductClient, cls).__new__(cls)
            cls._instance.available_services = []
            cls._instance.metric_type = "cosine"
            cls._instance.url_embedding = "http://10.0.1.247:8090/v1/urlencoding"
            cls._instance.url_detect = "http://10.0.1.247:8090/v1/det"
            cls._instance.product_embedding = ProductEmbeddingClient(domain=domain,
                                                                     port=port,
                                                                     api_key=api_key)

            cls._instance.chromadb_manager = ChromaDbManager()
            super(ProductClient, cls._instance).__init__(api_key, options)
        return cls._instance

    def __init__(self, api_key: str, domain: str, port: str, options: AllOptionsDict):
        super().__init__(api_key, options)  # Call the superclass constructor

    def get_available_functions(self) -> List[str]:
        """
        Get List of available functions in service
        :return:
        """
        return self.available_services

    def create_product_collection(self, collection_name):
        try:
            if self.chromadb_manager.check_collection_exists(collection_name):
                return 'Collection %s is already exists' % collection_name
            else:
                collection = self.chromadb_manager.create_collection(name=collection_name,
                                                                     metadata={"hnsw:space": self.metric_type})
                return collection
        except Exception as e:
            raise TypeError(e)

    def remove_example(self, collection_name, remove_id):
        try:
            if self.chromadb_manager.check_collection_exists(collection_name):
                collection = self.chromadb_manager.load_collection(
                    collection_name)
                collection.delete(
                    where={'image_id': remove_id}
                )
                return {'status': 'completed', 'result': {'image_id': remove_id}}
            else:
                return {'status': 'completed', 'error': 'Collection %s does not exists' % collection_name}
        except Exception as e:
            # raise TypeError(e)
            return {'status': 'completed', 'error': e}

    def list_examples(self, collection_name):
        try:
            if self.chromadb_manager.check_collection_exists(collection_name):
                collection = self.chromadb_manager.load_collection(
                    collection_name)

                collection_datas = collection.get(
                    include=["metadatas"]
                )
                result: dict = {'status': 'completed', 'result': pd.DataFrame(collection_datas['metadatas'], columns=[
                    'image_id', 'product_id', 'product_name']).to_dict('records')}
                return result
            else:
                return {'status': 'failed', 'error': 'Collection %s does not exists' % collection_name}
        except Exception as e:
            return {'status': 'failed', 'result': e}
            # raise TypeError(e)

    def remove_product_by_id(self, collection_name, remove_id):
        try:
            if self.chromadb_manager.check_collection_exists(collection_name):
                collection = self.chromadb_manager.load_collection(
                    collection_name)
                collection.delete(
                    where={'product_id': remove_id}
                )
                return {'status': 'completed', 'result': {'product_id': remove_id}}
            else:
                return {'status': 'completed', 'error': 'Collection %s does not exists' % collection_name}
        except Exception as e:
            # raise TypeError(e)
            return {'status': 'completed', 'error': e}

    def remove_product_by_name(self, collection_name, remove_name):
        try:
            if self.chromadb_manager.check_collection_exists(collection_name):
                collection = self.chromadb_manager.load_collection(
                    collection_name)
                collection.delete(
                    where={'product_name': remove_name}
                )
                return {'status': 'completed', 'result': {'product_name': remove_name}}
            else:
                return {'status': 'completed', 'error': 'Collection %s does not exists' % collection_name}
        except Exception as e:
            # raise TypeError(e)
            return {'status': 'completed', 'error': e}

    def remove_all_product(self, collection_name):
        try:
            if self.chromadb_manager.check_collection_exists(collection_name):
                number_of_products = self.chromadb_manager.count_row_of_collection(
                    collection_name=collection_name)
                collection = self.chromadb_manager.load_collection(
                    collection_name=collection_name)
                search_results = collection.query(
                    query_embeddings=[1]*512,
                    n_results=number_of_products,
                )

                for i in range(0, number_of_products, 40000):
                    collection.delete(
                        ids=search_results['ids'][0][i:i+40000]
                    )

                return {'status': 'completed', 'result': {'deleted': number_of_products}}
            else:
                return {'status': 'failed', 'error': 'Collection %s does not exists' % collection_name}
        except Exception as e:
            # raise TypeError(e)
            return {'status': 'failed', 'error': e}

    def remove_all_example_of_product_by_name(self, collection_name: str, product_name: str):
        try:
            if self.chromadb_manager.check_collection_exists(collection_name):
                number_of_examples = self.chromadb_manager.count_row_of_collection(
                    collection_name=collection_name)

                collection = self.chromadb_manager.load_collection(
                    collection_name=collection_name)

                collection.delete(
                    where={'product_name': product_name}
                )
                return {'status': 'completed', 'result': {'product_name': product_name, 'number_of_examples': number_of_examples}}
            else:
                return {'status': 'failed', 'error': 'Collection %s does not exists' % collection_name}
        except Exception as e:
            # raise TypeError(e)
            return {'status': 'failed', 'error': e}

    def list_products(self, collection_name):
        try:
            if self.chromadb_manager.check_collection_exists(collection_name):
                collection = self.chromadb_manager.load_collection(
                    collection_name)
                collection_datas = collection.get(
                    include=["metadatas"]
                )
                result = pd.DataFrame(collection_datas['metadatas'], columns=[
                    'product_id', 'product_name']).drop_duplicates(
                    subset=['product_id']).to_dict('records')

                return {'status': 'completed', 'result': result}
            else:
                return {'status': 'failed', 'error': 'Collection %s does not exists' % collection_name}
        except Exception as e:
            # raise TypeError(e)
            return {'status': 'failed', 'error': e}

    def add_data(self, collection_name,
                 product_ids, product_names, image_ids, embeddings):
        try:
            data = pd.DataFrame({
                "image_id": image_ids, "product_id": product_ids,
                "product_name": product_names
            })

            collection = self.chromadb_manager.load_or_create_collection(
                collection_name)

            collection.delete(
                where={'image_id': image_ids}
            )
            metadata = data.to_dict('records')

            collection.add(
                embeddings=embeddings,
                metadatas=metadata,
                ids=data['image_id'],
            )
            return ('Add  complete.')
        except Exception as e:
            raise TypeError(e)

    def add_example(self, collection_name,
                    product_id, product_name, image_id, url_image):
        try:

            response = self.product_embedding.post(
                image=url_image)
            data = pd.DataFrame({
                "image_id": [image_id], "product_id": [product_id],
                "product_name": [product_name]
            })
            collection = self.chromadb_manager.load_or_create_collection(
                collection_name)
            collection.delete(
                where={'image_id': image_id}
            )
            metadata = data.to_dict('records')
            collection.add(
                embeddings=response['results']['upload_products'],
                metadatas=metadata,
                ids=data['image_id'][0],
            )
            return {'status': 'completed',
                    'result': {'product_id': product_id,
                               'product_name': product_name,
                               'image_id': image_id}}
        except Exception as e:
            return {'status': 'failed', 'error': e}

    def add_data_from_csv(self, collection_name, csv_dir):
        for single_file in os.listdir(csv_dir):
            df = pd.read_csv(csv_dir + single_file)
            embeddings = [eval(vec) for vec in df.vector.values]
            data = pd.DataFrame({
                "image_id": df['imageID'], "product_id": df["productID"],
                "product_name": df["productName"]
            })
            collection = self.chromadb_manager.load_or_create_collection(
                collection_name)
            collection.delete(
                where={'image_id': df['imageID']}
            )
            metadata = data.to_dict('records')
            collection.add(
                embeddings=embeddings,
                metadatas=metadata,
                ids=data['image_id'].values.tolist(),
            )

    def add_product(self, collection_name: str, product_id: str,
                    product_name: str, image_ids: list, image_urls: list):
        try:
            embeddings = []
            collection = self.chromadb_manager.load_or_create_collection(
                collection_name)
            for image_url in image_urls:
                response = self.product_embedding.post(
                    image=image_url)
                embeddings.append(
                    response['results']['upload_products'])
            num_example = len(image_ids)
            data = pd.DataFrame({
                "image_id": image_ids, "product_id": [product_id] * num_example,
                "product_name": [product_name] * num_example
            })
            collection.delete(ids=image_ids)
            metadata = data.to_dict('records')
            collection.add(
                embeddings=embeddings,
                metadatas=metadata,
                ids=image_ids,
            )
            return {
                "status": 'completed',
                'result': {"image_id": image_ids,
                           "product_id": product_id,
                           "product_name": product_name}
            }
        except Exception as e:
            return {"status": 'failed', 'error': e}

    def rename_product_by_name(self, collection_name,
                       product_name, new_product_name):
        try:
            if self.chromadb_manager.check_collection_exists(collection_name):
                collection = self.chromadb_manager.load_collection(
                    collection_name)
                search_results = collection.query(
                    query_embeddings=[1]*512,
                    n_results=self.chromadb_manager.count_row_of_collection(
                        collection_name=collection_name),
                    where={"product_name": product_name},
                )
                collection.update(
                    ids=search_results['ids'][0],
                    metadatas=[{
                        'product_name': new_product_name}] * len(search_results['ids'][0])
                )
                return {
                    "status": 'completed', 'result': {"updated": True, "product_name": new_product_name}
                }
            else:
                return {"status": 'failed', 'result': {"updated": False,
                                                       'error': 'Collection %s does not exists' % collection_name}}
        except Exception as e:
            return {"status": 'failed', 'result': {"updated": False, 'error': e}}
        
    def rename_product_by_id(self, collection_name,
                       product_id, new_product_name):
        try:
            if self.chromadb_manager.check_collection_exists(collection_name):
                collection = self.chromadb_manager.load_collection(
                    collection_name)
                search_results = collection.query(
                    query_embeddings=[1]*512,
                    n_results=self.chromadb_manager.count_row_of_collection(
                        collection_name=collection_name),
                    where={"product_id": product_id},
                )
                collection.update(
                    ids=search_results['ids'][0],
                    metadatas=[{
                        'product_name': new_product_name}] * len(search_results['ids'][0])
                )
                return {
                    "status": 'completed', 'result': {"updated": True, "product_name": new_product_name}
                }
            else:
                return {"status": 'failed', 'result': {"updated": False,
                                                       'error': 'Collection %s does not exists' % collection_name}}
        except Exception as e:
            return {"status": 'failed', 'result': {"updated": False, 'error': e}}

    # def verify(self, collection_name, product_id, base64_image):
    #     try:
    #         payload = {'base64_image': base64_image}
    #         headers = {}
    #         response = requests.request(
    #             "POST", self.url_detect, headers=headers, data=payload)

    #         if self.chromadb_manager.check_collection_exists(collection_name):
    #             collection = self.chromadb_manager.load_collection(
    #                 collection_name)

    #             search_result = collection.query(
    #                 query_embeddings=response.data.uploaded_products,
    #                 n_results=1,
    #                 where={"product_id": product_id},
    #             )

    #             if (1 - search_result[0]['_distance'] > 0.52):
    #                 return {'result': 'True'}
    #             else:
    #                 return {'result': 'False'}
    #         else:
    #             return 'Collection %s does not exists' % collection_name
    #     except Exception as e:
    #         raise TypeError(e)

    # def verify_two_image(self, collection_name, subject_id, source_image, target_image):
    #     try:
    #         payload = {'source_image': source_image,
    #                    'target_image': target_image}
    #         headers = {}
    #         response = requests.request(
    #             "POST", self.url_embedding, headers=headers, data=payload)

    #         if self.chromadb_manager.check_collection_exists(collection_name):
    #             collection = self.chromadb_manager.load_collection(
    #                 collection_name)

    #             search_result = collection.query(
    #                 query_embeddings=response.data.uploaded_faces,
    #                 n_results=1,
    #             )

    #             if (1 - search_result[0]['_distance'] > 0.52):
    #                 return {'result': 'True'}
    #             else:
    #                 return {'result': 'False'}
    #         else:
    #             return 'Collection %s does not exists' % collection_name
    #     except Exception as e:
    #         raise TypeError(e)

    # def count_1(self, collection_name, search_embeddings):
    #     try:

    #         if self.chromadb_manager.check_collection_exists(collection_name):
    #             search_results = []
    #             product_names = []
    #             results = []
    #             collection = self.chromadb_manager.load_collection(
    #                 collection_name)

    #             search_results = collection.query(
    #                 query_embeddings=search_embeddings,
    #                 n_results=1,
    #             )

    #             for search_result in search_results:
    #                 simility = 1 - search_result[0]['_distance']
    #                 if simility > 0.52:
    #                     product_names.append(search_results[0]['product_name'])
    #                     dict_res = {
    #                         'label': search_results[0]['product_name'],
    #                         'score': simility
    #                     }
    #                     results.append(dict_res)
    #                 else:
    #                     product_names.append("Unknow")
    #                     dict_res = {'label': "Unknow", 'score': simility}
    #             result_dicts = {'count': dict(
    #                 collections.Counter(product_names)), 'locate': results}
    #             return result_dicts
    #         else:
    #             return 'Collection %s does not exists' % collection_name
    #     except Exception as e:
    #         raise TypeError(e)

    # def count(self, collection_name, base64_image):
    #     try:
    #         payload = {'base64_image': base64_image}
    #         headers = {}
    #         response = requests.request(
    #             "POST", self.url_detect, headers=headers, data=payload)
    #         search_embeddings = response.data.uploaded_products
    #         det_results = response.data.det_results

    #         if self.chromadb_manager.check_collection_exists(collection_name):
    #             search_results = []
    #             product_names = []
    #             # results = []
    #             collection = self.chromadb_manager.load_collection(
    #                 collection_name)
    #             search_results = collection.query(
    #                 query_embeddings=search_embeddings,
    #                 n_results=1,
    #             )
    #             # for embedding in search_embeddings:
    #             #     search_result = collection.search(embedding) \
    #             #         .metricType(self.metric_type) \
    #             #         .limit(1) \
    #             #         .to_list()
    #             #     search_results.append(search_result)

    #             size_seach_results = len(search_results)
    #             for i in range(size_seach_results):
    #                 simility = 1 - search_results[i][0]['_distance']
    #                 if simility > 0.52:
    #                     product_names.append(
    #                         search_results[i][0]['product_name'])
    #                     det_results[i]['score'] = simility
    #                     det_results[i]['label'] = search_results[i][0]['product_name']
    #                 else:
    #                     product_names.append("Unknow")
    #                     det_results[i]['score'] = simility
    #                     det_results[i]['label'] = 'Unknow'
    #             result_dicts = {'count': dict(collections.Counter(
    #                 product_names)), 'locate': det_results}
    #             return result_dicts
    #         else:
    #             return 'Collection %s does not exists' % collection_name
    #     except Exception as e:
    #         raise TypeError(e)

    # def detect(self, base64_image):
    #     try:
    #         payload = {'base64_image': base64_image}
    #         headers = {}
    #         response = requests.request(
    #             "POST", self.url_detect, headers=headers, data=payload)
    #         return response
    #     except Exception as e:
    #         raise TypeError(e)

    # def embedding(self, base64_image):
    #     try:
    #         payload = {'base64_image': base64_image}
    #         headers = {}
    #         response = requests.request(
    #             "POST", self.url_embedding, headers=headers, data=payload)
    #         return response
    #     except Exception as e:
    #         raise TypeError(e)
