#
# product_count.py
#
# Created by Nguyen Quang Duy on Tue Dec 19 2023
#

import requests

from deepvision.common.multipart_constructor import multipart_constructor
from deepvision.common.typed_dict import ExpandedOptionsDict, check_fields_by_name
from deepvision.common import ClientRequest
from deepvision.chroma_manager import ChromaDbManager
from deepvision.client.product.list_product_embedding import ListProductEmbeddingClient
import collections


class ProductCountClient(ClientRequest):
    """
        Count product in image. It uses source and target images for encode and send to DeepVision 
        server.
    """

    def __init__(self, api_key: str, domain: str, port: str):
        super().__init__()
        self.product_embedding: ListProductEmbeddingClient = ListProductEmbeddingClient(
            domain=domain,
            port=port,
            api_key=api_key
        )
        self.chromadb_manager: ChromaDbManager = ChromaDbManager()

    def get(self):
        pass

    def post(self,
             collection_name: str = '',
             image: str = '' or bytes,
             options: ExpandedOptionsDict = {}) -> dict:
        """
            count product in image. 
            :collection_name: name of collection product
            :param image: Path to source image in file system.
            :param target_image: Path to target image in file system.
            :param image_id: product id from previously added image. 
            :param options: dictionary with options for server.

            :return: json from server.
        """
        try:
            response = self.product_embedding.post(
                image)
            search_embeddings = response['results']['upload_products']
            det_results = response['results']['locates']

            if self.chromadb_manager.check_collection_exists(collection_name):
                search_results = []
                product_names = []

                collection = self.chromadb_manager.load_collection(
                    collection_name)
                search_results = collection.query(
                    query_embeddings=search_embeddings,
                    n_results=1,
                )

                size_seach_results = len(search_results)
                for i in range(size_seach_results):
                    simility = 1 - search_results['distances'][i][0]
                    if simility > 0.52:
                        product_names.append(
                            search_results['metadatas'][i][0]['product_name'])
                        det_results[i]['score'] = simility
                        det_results[i]['label'] = search_results['metadatas'][i][0]['product_name']
                    else:
                        product_names.append("Unknow")
                        det_results[i]['score'] = simility
                        det_results[i]['label'] = 'Unknow'
                result_dicts = {'count': dict(collections.Counter(
                    product_names)), 'locate': det_results}
                return {
                    'status': 'completed',
                    'result': result_dicts
                }
            else:
                return {
                    'status': 'completed',
                    'error': 'Collection %s does not exists' % collection_name
                }
        except Exception as e:
            return {
                'status': 'failed',
                'error': e
            }

    def put(self):
        pass

    def delete(self):
        pass