#
# product_verify.py
#
# Created by Nguyen Quang Duy on Tue Dec 19 2023
#

import requests

from deepvision.common.multipart_constructor import multipart_constructor
from deepvision.common.typed_dict import ExpandedOptionsDict, check_fields_by_name
from deepvision.common import ClientRequest
from deepvision.chroma_manager import ChromaDbManager
from deepvision.client.product import ProductEmbeddingClient


class ProductRecognizeClient(ClientRequest):
    """
        Recogni product in image. It uses source and target images for encode and send to DeepVision 
        server.
    """

    def __init__(self, api_key: str, domain: str, port: str):
        super().__init__()
        self.product_embedding: ProductEmbeddingClient = ProductEmbeddingClient(
            domain=domain,
            port=port,
            api_key=api_key
        )
        self.chromadb_manager: ChromaDbManager = ChromaDbManager()

    def get(self):
        pass

    def post(self,
             collection_name: str = '',
             image: str = '' or bytes,
             options: ExpandedOptionsDict = {}) -> dict:
        """
            recognize product in image. 
            :collection_name: name of collection product
            :param image: Path to source image in file system.
            :param target_image: Path to target image in file system.
            :param image_id: product id from previously added image. 
            :param options: dictionary with options for server.

            :return: json from server.
        """
        try:
            response = self.product_embedding.post(
                image)

            if self.chromadb_manager.check_collection_exists(collection_name):
                collection = self.chromadb_manager.load_collection(
                    collection_name)

                search_results = collection.query(
                    query_embeddings=response['results']['upload_products'],
                    n_results=1,
                )
                similarity = 1 - search_results['distances'][0][0]
                if (similarity > 0.52):
                    return {
                        'status': 'completed',
                        'result': {
                            'embedding': response['results']['upload_products'],
                            'bbox': response['results']['locates'],
                            'products': {
                                "similarity": similarity,
                                "product": search_results['metadatas'][0][0]['product_name']
                            }
                        }
                    }
                else:
                    return {
                        'status': 'completed',
                        'result': {
                            'embedding': response['results']['upload_products'],
                            'bbox': response['results']['locates'],
                            'products': {
                                "similarity": similarity,
                                "product": 'Unknow'
                            }
                        }
                    }
            else:
                return {
                    'status': 'completed',
                    'error': 'Collection %s does not exists' % collection_name
                }
        except Exception as e:
            return {
                'status': 'failed',
                'error': e
            }

    def put(self):
        pass

    def delete(self):
        pass
