#
# product_verification.py
#
# Created by Nguyen Quang Duy on Tue Dec 19 2023
#

import requests

from deepvision.common.multipart_constructor import multipart_constructor
from deepvision.common.typed_dict import ExpandedOptionsDict, check_fields_by_name
from deepvision.common import ClientRequest
from deepvision.chroma_manager import ChromaDbManager
from deepvision.client.product import ProductEmbeddingClient


class ProductVerificationClient(ClientRequest):
    """
        Compare product in image. It uses image path for encode and validation by product id.
    """

    def __init__(self, api_key: str, domain: str, port: str):
        super().__init__()
        self.product_embedding: ProductEmbeddingClient = ProductEmbeddingClient(
            domain=domain,
            port=port,
            api_key=api_key
        )
        self.chromadb_manager: ChromaDbManager = ChromaDbManager()

    def get(self) -> dict:
        pass

    """
        POST request for compare face in image using image id. 
        
        :param image_path: Path to image in file system.
        :param image_id: subject id from previously added image.
        :param options: dictionary with options for server.
        
        :return: json from server.
    """

    def post(self,
             collection_name: str = '',
             image: str = '' or bytes,
             product_id: str = '',
             options: ExpandedOptionsDict = {}) -> dict:
        try:
            response = self.product_embedding.post(
                image)
            if self.chromadb_manager.check_collection_exists(collection_name):
                collection = self.chromadb_manager.load_collection(
                    collection_name)
                search_result = collection.query(
                    query_embeddings=response['results']['upload_products'],
                    n_results=1,
                    where={"product_id": product_id},
                )
                product_name = search_result['metadatas'][0][0]['product_name']
                if (1 - search_result['distances'][0][0] > 0.52):
                    return {
                        'status': 'completed',
                        'result': {
                            'verify': True,
                            'product': product_name,
                            'embedding': response['results']['upload_products'],
                            'bbox': response['results']['locates']
                        }

                    }
                else:
                    return {
                        'status': 'completed',
                        'result': {
                            'verify': False,
                            'product': product_name,
                            'embedding': response['results']['upload_products'],
                            'bbox': response['results']['locates']
                        }

                    }
            else:
                return {
                    'status': 'failed',
                    'error': 'Collection %s does not exists' % collection_name
                }
        except Exception as e:
            return {
                'status': 'failed',
                'error': e
            }

    def put(self):
        pass

    def delete(self):
        pass
