#
# product_verify.py
#
# Created by Nguyen Quang Duy on Tue Dec 19 2023
#

import requests
import numpy as np
from deepvision.common.multipart_constructor import multipart_constructor_with_two_images
from deepvision.common.typed_dict import ExpandedOptionsDict, check_fields_by_name
from deepvision.common import ClientRequest
from deepvision.config.api_list import PRODUCT_EMBEDDING_API
from deepvision.utils.verify import verify
from deepvision.client.product import ProductEmbeddingClient


class ProductVerifyClient(ClientRequest):
    """
        Verify product in image. It uses source and target images.
    """

    def __init__(self, api_key: str, domain: str, port: str):
        super().__init__()
        self.product_embedding: ProductEmbeddingClient = ProductEmbeddingClient(
            domain=domain,
            port=port,
            api_key=api_key
        )

    def get(self):
        pass

    def post(self,
             source_image: str = '' or bytes,
             target_image: str = '' or bytes,
             options: ExpandedOptionsDict = {}) -> dict:
        """
            POST request for verify product in image using source and target images. 

            :param source_image: Path to source image in file system.
            :param target_image: Path to target image in file system.
            :param image_id: product id from previously added image. 
            :param options: dictionary with options for server.

            :return: json.
        """
        try:
            source_image_product_response = self.product_embedding.post(source_image)
            matches_image_product_response = self.product_embedding.post(target_image)
            result, distances = verify([source_image_product_response['results']['upload_products']],
                            [matches_image_product_response['results']['upload_products']])
            return {
                'status': 'completed',
                'result': {
                           'source_image_product': {'embedding': source_image_product_response['results']['upload_products'],
                                                    'bbox': source_image_product_response['results']['locates']},
                           'product_matches': {'embedding': matches_image_product_response['results']['upload_products'],
                                               'bbox': source_image_product_response['results']['locates'],
                                               'verify': result,
                                               "similarity" : distances
                                               }}
            }
        except Exception as e:
            return {
                'status': 'failed',
                'error': e
            }

    def put(self):
        pass

    def delete(self):
        pass
