#
# product_collections.py
#
# Created by Nguyen Quang Duy on Sat Dec 16 2023
#

from deepvision.common.typed_dict import AllOptionsDict, ExpandedOptionsDict, DetProbOptionsDict, pass_dict
from deepvision.use_cases.product import (
    AddExampleOfProduct,
    AddProduct,
    DeleteAllExamplesOfProductByName,
    DeleteProductByName,
    DeleteProductByID,
    DeleteAllProducts,
    DeleteExampleById,
    GetProducts,
    UpdateProductById,
    UpdateProductByName,
    VerificationProductFromImage,
    ListOfAllSavedProducts
)


class ProductCollection:
    def __init__(self, api_key: str, domain: str, port: str, options: AllOptionsDict = {}):
        """Init service with define API Key"""
        self.available_services = []
        self.api_key = api_key
        self.options = options
        self.add_example: AddExampleOfProduct = AddExampleOfProduct(
            domain=domain,
            port=port,
            api_key=api_key
        )
        self.list_of_all_saved_products: ListOfAllSavedProducts = ListOfAllSavedProducts(
            domain=domain,
            port=port,
            api_key=api_key
        )

        self.delete_all_examples_of_product_by_name: DeleteAllExamplesOfProductByName = DeleteAllExamplesOfProductByName(
            domain=domain,
            port=port,
            api_key=api_key
        )
        self.delete_all_examples_by_id: DeleteExampleById = DeleteExampleById(
            domain=domain,
            port=port,
            api_key=api_key
        )
        self.verify_product_from_image: VerificationProductFromImage = VerificationProductFromImage(
            domain=domain,
            port=port,
            api_key=api_key
        )

    def list(self, collection_name:str) -> dict:
        """
        Get list of collections
        :return:
        """
        request = ListOfAllSavedProducts.Request(
            collection_name=collection_name
        )
        return self.list_of_all_saved_products.execute(request)

    def add(self, collection_name:str, image_id: str, image_path: str, product_id:str, product_name: str, options: DetProbOptionsDict = {}) -> dict:
        """
        Add example to collection
        :param image_path:
        :param product:
        :return:
        """
        request = AddExampleOfProduct.Request(
            collection_name=collection_name,
            api_key=self.api_key,
            image_path=image_path,
            image_id=image_id,
            product_id=product_id,
            product_name=product_name
        )
        return self.add_example.execute(request, pass_dict(options, DetProbOptionsDict) if options == {} else options)

    def delete(self, collection_name:str, image_id: str) -> dict:
        """
        Delete example by Id
        :param image_id:
        :return:
        """
        request = DeleteExampleById.Request(
            api_key=self.api_key,
            collection_name=collection_name,
            image_id=image_id
        )
        return self.delete_all_examples_by_id.execute(request)

    def delete_all(self, collection_name:str, product_name: str) -> dict:
        """
        Delete all examples of product
        :param product:
        :return:
        """
        request = DeleteAllExamplesOfProductByName.Request(
            api_key=self.api_key,
            collection_name=collection_name,
            product_name=product_name
        )
        return self.delete_all_examples_of_product_by_name.execute(request)

    def verify(self, collection_name:str, image_path: str, product_id: str, options: ExpandedOptionsDict = {}) -> dict:
        """
        Compare image
        :param collection_name:
        :param image_path:
        :param product_id:
        :return:
        """
        request = VerificationProductFromImage.Request(
            api_key=self.api_key,
            collection_name=collection_name,
            image_path=image_path,
            product_id=product_id
        )
        return self.verify_product_from_image.execute(request, pass_dict(options, ExpandedOptionsDict) if options == {} else options)


class Products:
    def __init__(self, api_key: str, domain: str, port: str, options: AllOptionsDict = {}):
        """Init service with define API Key"""
        self.available_services = []
        self.api_key = api_key
        self.options = options
        self.add_product: AddProduct = AddProduct(
            domain=domain,
            port=port,
            api_key=api_key
        )
        self.update_product_by_id: UpdateProductById = UpdateProductById(
            domain=domain,
            port=port,
            api_key=api_key
        )
        self.update_product_by_name: UpdateProductByName = UpdateProductByName(
            domain=domain,
            port=port,
            api_key=api_key
        )
        self._delete_product_by_name: DeleteProductByName = DeleteProductByName(
            domain=domain,
            port=port,
            api_key=api_key
        )
        self._delete_product_by_id: DeleteProductByID = DeleteProductByID(
            domain=domain,
            port=port,
            api_key=api_key
        )
        self.delete_all_products: DeleteAllProducts = DeleteAllProducts(
            domain=domain,
            port=port,
            api_key=api_key
        )
        self.list_of_all_saved_products: GetProducts = GetProducts(
            domain=domain,
            port=port,
            api_key=api_key
        )

    def list(self, collection_name: str) -> dict:
        """
        Get list of products
        :return:
        """
        request = GetProducts.Request(
            collection_name=collection_name,
        )
        return self.list_of_all_saved_products.execute(request)

    def add(self, collection_name:str, product_id: str, product_name:str, image_id: list, image_path: list) -> dict:
        """
        Add product
        :param product:
        :return:
        """
        request = AddProduct.Request(
            collection_name=collection_name,
            image_path=image_path,
            product_id=product_id,
            product_name=product_name,
            image_id=image_id
        )
        return self.add_product.execute(request)

    def update_by_name(self, collection_name: str, product_name: str, new_name: str) -> dict:
        """
        Update product by name
        :param product_name:
        :param new_name:
        :return:
        """
        request = UpdateProductByName.Request(
            collection_name=collection_name,
            product_name=product_name,
            new_product_name=new_name
        )
        return self.update_product_by_name.execute(request)
    
    def update_by_id(self, collection_name: str, product_id: str, new_name: str) -> dict:
        """
        Update product by id
        :param product_id:
        :param new_name:
        :return:
        """
        request = UpdateProductById.Request(
            collection_name=collection_name,
            product_id=product_id,
            new_product_name=new_name
        )
        return self.update_product_by_id.execute(request)

    def delete_product_by_name(self, collection_name: str, product_name: str) -> dict:
        """
        Delete product by name
        :param product name:
        :return:
        """
        request = DeleteProductByName.Request(
            collection_name=collection_name,
            product_name=product_name
        )
        return self._delete_product_by_name.execute(request)
    
    def delete_product_by_id(self, collection_name: str, product_name: str) -> dict:
        """
        Delete product by id
        :param product id:
        :return:
        """
        request = DeleteProductByID.Request(
            collection_name=collection_name,
            product_name=product_name
        )
        return self._delete_product_by_id.execute(request)

    def delete_all(self, collection_name: str) -> dict:
        """
        Delete all products
        :return:
        """
        return self.delete_all_products.execute(collection_name)
