#
# model.py
#
# Created by Nguyen Quang Duy on Sat Dec 16 2023
#

from typing import Optional
from deepvision.common.typed_dict import AllOptionsDict
from deepvision.config import config
from deepvision.service import (FaceDetectionService,
                     FaceRecognitionService,
                     FaceVerificationService,
                     FaceVerifyService,
                     ProductRecognitionService,
                     ProductCountService,
                     ProductDetectionService,
                     ProductVerificationService,
                     ProductVerifyService,
                     PowerMeterService,
                     WaterMeterService,
                     PPEService,
                     ImageTaggingService,
                     OCRService
                     )


class DeepVision(object):
    """
    Main class
    """

    def __init__(self, domain: str = config.DOMAIN, port: str = config.PORT, options: AllOptionsDict = {}):
        self._domain: str = domain
        self._port: str = port
        self._options: AllOptionsDict = options
        self.face_detection: Optional[FaceDetectionService] = None
        self.face_recognition: Optional[FaceRecognitionService] = None
        self.face_verify: Optional[FaceVerifyService] = None
        self.face_verification: Optional[FaceVerificationService] = None
        self.product_detection: Optional[ProductDetectionService] = None
        self.product_recognition: Optional[ProductRecognitionService] = None
        self.product_verify: Optional[ProductVerifyService] = None
        self.product_verification: Optional[ProductVerificationService] = None
        self.product_count: Optional[ProductCountService] = None
        self.powermeter: Optional[PowerMeterService] = None
        self.watermeter: Optional[WaterMeterService] = None
        self.ppe: Optional[PPEService] = None
        self.imagetagging: Optional[ImageTaggingService] = None
        self.ocr: Optional[OCRService] = None

    @property
    def domain(self):
        return self._domain

    @domain.setter
    def domain(self, domain: str):
        self._domain = domain

    @property
    def port(self):
        return self._port

    @port.setter
    def port(self, port: str):
        self._port = port

    @property
    def options(self):
        return self._options

    @options.setter
    def options(self, options: AllOptionsDict):
        self._options = options

    def init_face_detection_service(self, api_key: str) -> FaceDetectionService:
        """
        Init Face Detection Service
        :param api_key:
        :return:
        """
        self.face_detection = FaceDetectionService(api_key=api_key,
                                domain=self.domain,
                                port=self.port,
                                options=self.options
                                )
        return self.face_detection
    
    
    def init_face_recognition_service(self, api_key: str) -> FaceRecognitionService:
        """
        Init Face Recognition Service
        :param api_key:
        :return:
        """
        self.face_recognition = FaceRecognitionService(api_key=api_key,
                                domain=self.domain,
                                port=self.port,
                                options=self.options
                                )
        return self.face_recognition
    
    def init_face_verification_service(self, api_key: str) -> FaceVerificationService:
        """
        Init Face verification Service
        :param api_key:
        :return:
        """
        self.face_verification = FaceVerificationService(api_key=api_key,
                                domain=self.domain,
                                port=self.port,
                                options=self.options
                                )
        return self.face_verification
    
    def init_face_verify_service(self, api_key: str) -> FaceVerifyService:
        """
        Init Face Verify Service
        :param api_key:
        :return:
        """
        self.face_verify = FaceVerifyService(api_key=api_key,
                                domain=self.domain,
                                port=self.port,
                                options=self.options
                                )
        return self.face_verify

    def init_product_count_service(self, api_key: str) -> ProductCountService:
        """
        Init Product Count Service
        :param api_key:
        :return:
        """
        self.product_count = ProductCountService(api_key=api_key,
                                      domain=self.domain,
                                      port=self.port,
                                      options=self.options)
        return self.product_count
    
    def init_product_recognition_service(self, api_key: str) -> ProductRecognitionService:
        """
        Init Product Recognition Service
        :param api_key:
        :return:
        """
        self.product_recognition = ProductRecognitionService(api_key=api_key,
                                      domain=self.domain,
                                      port=self.port,
                                      options=self.options)
        return self.product_recognition
    
    def init_product_detection_service(self, api_key: str) -> ProductDetectionService:
        """
        Init Product Detection Service
        :param api_key:
        :return:
        """
        self.product_detection = ProductDetectionService(api_key=api_key,
                                      domain=self.domain,
                                      port=self.port,
                                      options=self.options)
        return self.product_detection
    
    def init_product_verification_service(self, api_key: str) -> ProductVerificationService:
        """
        Init Product Verification Service
        :param api_key:
        :return:
        """
        self.product_verification = ProductVerificationService(api_key=api_key,
                                      domain=self.domain,
                                      port=self.port,
                                      options=self.options)
        return self.product_verification
    
    def init_product_verify_service(self, api_key: str) -> ProductVerifyService:
        """
        Init Product Verify Service
        :param api_key:
        :return:
        """
        self.product_verify = ProductVerifyService(api_key=api_key,
                                      domain=self.domain,
                                      port=self.port,
                                      options=self.options)
        return self.product_verify

    def init_ocr_service(self, api_key: str) -> OCRService:
        """
        Init OCR Service
        :param api_key:
        :return:
        """
        self.ocr = OCRService(api_key=api_key,
                              domain=self.domain,
                              port=self.port,
                              options=self.options)
        return self.ocr

    def init_powermeter_service(self, api_key: str) -> PowerMeterService:
        """
        Init Power Meter Service
        :param api_key:
        :return:
        """
        self.powermeter = PowerMeterService(api_key=api_key,
                                            domain=self.domain,
                                            port=self.port,
                                            options=self.options)
        return self.powermeter

    def init_watermeter_service(self, api_key: str) -> WaterMeterService:
        """
        Init Water Meter Service
        :param api_key:
        :return:
        """
        self.watermeter = WaterMeterService(api_key=api_key,
                                            domain=self.domain,
                                            port=self.port,
                                            options=self.options)
        return self.watermeter

    def init_ppe_service(self, api_key: str) -> PPEService:
        """
        Init PPE Service
        :param api_key:
        :return:
        """
        self.ppe = PPEService(dapi_key=api_key,
                              domain=self.domain,
                              port=self.port,
                              options=self.options)
        return self.ppe

    def init_imagetagging_service(self, api_key: str) -> ImageTaggingService:
        """
        Init Image Tagging Service
        :param api_key:
        :return:
        """
        self.imagetagginga = ImageTaggingService(api_key=api_key,
                                                 domain=self.domain,
                                                 port=self.port,
                                                 options=self.options)
        return self.imagetagging
